@@ -473,7 +473,7 @@ text_input_enter(void *data,
 	if (surface != window_get_wl_surface(entry->window))
 		return;
 
-	entry->active = 1;
+	entry->active++;
 
 	text_entry_update(entry);
 	entry->reset_serial = entry->serial;
@@ -488,10 +488,10 @@ text_input_leave(void *data,
 	struct text_entry *entry = data;
 
 	text_entry_commit_and_reset(entry);
+	entry->active--;
 
-	entry->active = 0;
-
-	wl_text_input_hide_input_panel(text_input);
+	if (!entry->active)
+		wl_text_input_hide_input_panel(text_input);
 
 	widget_schedule_redraw(entry->widget);
 }
@@ -682,7 +682,7 @@ text_entry_update_layout(struct text_entry *entry)
 	       (entry->preedit.text ? strlen(entry->preedit.text) : 0)));
 
 	if (entry->preedit.text) {
-		text = malloc(strlen(entry->text) + strlen(entry->preedit.text) + 1);
+		text = xmalloc(strlen(entry->text) + strlen(entry->preedit.text) + 1);
 		strncpy(text, entry->text, entry->cursor);
 		strcpy(text + entry->cursor, entry->preedit.text);
 		strcpy(text + entry->cursor + strlen(entry->preedit.text),
@@ -764,7 +764,7 @@ static void
 text_entry_insert_at_cursor(struct text_entry *entry, const char *text,
 			    int32_t cursor, int32_t anchor)
 {
-	char *new_text = malloc(strlen(entry->text) + strlen(text) + 1);
+	char *new_text = xmalloc(strlen(entry->text) + strlen(text) + 1);
 
 	strncpy(new_text, entry->text, entry->cursor);
 	strcpy(new_text + entry->cursor, text);
