@@ -32,9 +32,9 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <xf86drm.h>
 #include <xf86drmMode.h>
 
 #include "libbacklight.h"
